/*
 * MainActivity.java
 * This file is part of ArduinoYunREST
 *
 * Copyright (C) 2013 - Andreas Rudolf
 *
 * ArduinoYunREST is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ArduinoYunREST is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ArduinoYunREST. If not, see <http://www.gnu.org/licenses/>.
 */
package ch.serverbox.android.arduino.rest;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.atomic.AtomicBoolean;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONObject;

import com.example.arduinoled.R;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;

/**
 * (c) Nexus-Computing GmbH Switzerland
 * @author Andreas Rudolf, 27.09.2013
 *
 * Use this Android application together with the official Arduino bridge sketch from here:
 * http://arduino.cc/en/Tutorial/Bridge
 * 
 * Make sure to adjust ARDUINO_IP_ADDRESS to the IP of your Arduino Yun
 *
 */
public class MainActivity extends Activity {

	// TODO: adjust ARDUINO_IP_ADDRESS
	public final String ARDUINO_IP_ADDRESS = "192.168.1.47";
	public final String TAG = "ArduinoYun";

	private SeekBar mSeekBar;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		mSeekBar = (SeekBar) findViewById(R.id.seekBar);
		mSeekBar.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {}
			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {}
			@Override
			public void onProgressChanged(SeekBar seekBar, int progress,
					boolean fromUser) {
				mQueue.offer(progress);
			}
		});
	}

	@Override
	protected void onStart() {
		mStop.set(false);
		if(sNetworkThreadSend == null){
			sNetworkThreadSend = new Thread(mNetworkRunnableSend);
			sNetworkThreadSend.start();
		}
		if(sNetworkThreadReceive == null){
			sNetworkThreadReceive = new Thread(mNetworkRunnableReceive);
			sNetworkThreadReceive.start();
		}
		super.onStart();
	}

	@Override
	protected void onStop() {
		mStop.set(true);
		mQueue.clear();
		mQueue.offer(-1);
		if(sNetworkThreadSend != null) sNetworkThreadSend.interrupt();
		if(sNetworkThreadReceive != null) sNetworkThreadReceive.interrupt();
		super.onStop();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	public void log(String s){
		Log.d(">==< "+TAG+" >==<", s);
	}

	private ArrayBlockingQueue<Integer> mQueue = new ArrayBlockingQueue<Integer>(100);
	private AtomicBoolean mStop = new AtomicBoolean(false);

	private static Thread sNetworkThreadSend = null;
	private final Runnable mNetworkRunnableSend = new Runnable() {

		@Override
		public void run() {
			log("starting network thread for sending");
			String urlBase = "http://"+ARDUINO_IP_ADDRESS+"/arduino/analog/13/";
			String url;

			try {
				while(!mStop.get()){
					int val = mQueue.take();
					if(val >= 0){
						HttpClient httpClient = new DefaultHttpClient();
						url = urlBase.concat(String.valueOf(val));
						log("executing httpClient request");
						HttpResponse response = httpClient.execute(new HttpGet(url));
					}
				}
			} catch (ClientProtocolException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

			log("returning from network thread for sending");
			sNetworkThreadSend = null;
		}
	};

	private static Thread sNetworkThreadReceive = null;
	private final Runnable mNetworkRunnableReceive = new Runnable() {

		@Override
		public void run() {
			log("starting network thread for receiving");

			String url = "http://"+ARDUINO_IP_ADDRESS+"/data/get/D13";

			while(!mStop.get()){
				try {
					String string = readURL(url);
					String value = "";
					try {
						JSONObject jsonMain = new JSONObject(string);
						value = jsonMain.get("value").toString();
						log("value = "+value);
						
					} catch (Exception e) {
						e.printStackTrace();
					}
					updateText(value);
					
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}

			log("returning from network thread for receiving");
			sNetworkThreadReceive = null;
		}
	};

	public void updateText(final String value){
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				TextView textView = (TextView) findViewById(R.id.value);
				textView.setText(value);
			}
		});
		
	}
	
	public String readURL(String value){
		URL url;
		StringBuilder builder = new StringBuilder();
		try {
			url = new URL(value);
			URLConnection yc = url.openConnection();
			BufferedReader in = new BufferedReader(new InputStreamReader(yc.getInputStream()));
			String inputLine;
			while ((inputLine = in.readLine()) != null){ 
				builder.append(inputLine);
			}
			in.close();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return builder.toString();
	}

}
